@@ -32,6 +32,7 @@
 #include <cairo/cairo-gobject.h>
 #include <vte/vte.h>
 #include "vtepcre2.h"
+#include "glib-glue.hh"
 
 #include <algorithm>
 
@@ -58,10 +59,11 @@ public:
         gboolean no_rewrap{false};
         gboolean no_shaping{false};
         gboolean no_shell{false};
+        gboolean no_systemd_scope{false};
         gboolean object_notifications{false};
+        gboolean require_systemd_scope{false};
         gboolean reverse{false};
         gboolean test_mode{false};
-        gboolean use_gregex{false};
         gboolean version{false};
         gboolean whole_window_transparent{false};
         bool bg_color_set{false};
@@ -371,8 +373,6 @@ public:
                           "Specify a font to use", nullptr },
                         { "foreground-color", 0, 0, G_OPTION_ARG_CALLBACK, (void*)parse_fg_color,
                           "Set default foreground color", "COLOR" },
-                        { "gregex", 0, 0, G_OPTION_ARG_NONE, &use_gregex,
-                          "Use GRegex instead of PCRE2", nullptr },
                         { "geometry", 'g', 0, G_OPTION_ARG_STRING, &geometry,
                           "Set the size (in characters) and position", "GEOMETRY" },
                         { "highlight-background-color", 0, 0, G_OPTION_ARG_CALLBACK, (void*)parse_hl_bg_color,
@@ -405,12 +405,16 @@ public:
                           "Disable Arabic shaping", nullptr },
                         { "no-shell", 'S', 0, G_OPTION_ARG_NONE, &no_shell,
                           "Disable spawning a shell inside the terminal", nullptr },
+                        { "no-systemd-scope", 0, 0, G_OPTION_ARG_NONE, &no_systemd_scope,
+                          "Don't use systemd user scope", nullptr },
                         { "object-notifications", 'N', 0, G_OPTION_ARG_NONE, &object_notifications,
                           "Print VteTerminal object notifications", nullptr },
                         { "output-file", 0, 0, G_OPTION_ARG_FILENAME, &output_filename,
                           "Save terminal contents to file at exit", nullptr },
                         { "reverse", 0, 0, G_OPTION_ARG_NONE, &reverse,
                           "Reverse foreground/background colors", nullptr },
+                        { "require-systemd-scope", 0, 0, G_OPTION_ARG_NONE, &require_systemd_scope,
+                          "Require use of a systemd user scope", nullptr },
                         { "scrollback-lines", 'n', 0, G_OPTION_ARG_INT, &scrollback_lines,
                           "Specify the number of scrollback-lines (-1 for infinite)", nullptr },
                         { "transparent", 'T', 0, G_OPTION_ARG_INT, &transparency_percent,
@@ -511,28 +515,15 @@ verbose_fprintf(FILE* fp,
 
 /* regex */
 
-static GRegex*
-compile_gregex(char const* pattern,
-               bool caseless,
-               GError** error)
-{
-        GRegexCompileFlags flags = GRegexCompileFlags(G_REGEX_OPTIMIZE |
-                                                      G_REGEX_MULTILINE |
-                                                      caseless ? G_REGEX_CASELESS : 0);
-
-        return g_regex_new(pattern, flags, GRegexMatchFlags(0), error);
-}
-
 static void
 jit_regex(VteRegex* regex,
           char const* pattern)
 {
-        GError *err = nullptr;
-        if (!vte_regex_jit(regex, PCRE2_JIT_COMPLETE, &err) ||
-            !vte_regex_jit(regex, PCRE2_JIT_PARTIAL_SOFT, &err)) {
-                if (!g_error_matches(err, VTE_REGEX_ERROR, -45 /* PCRE2_ERROR_JIT_BADOPTION: JIT not supported */))
-                        verbose_printerr("JITing regex \"%s\" failed: %s\n", pattern, err->message);
-                g_error_free(err);
+        auto error = vte::glib::Error{};
+        if (!vte_regex_jit(regex, PCRE2_JIT_COMPLETE, error) ||
+            !vte_regex_jit(regex, PCRE2_JIT_PARTIAL_SOFT, error)) {
+                if (!error.matches(VTE_REGEX_ERROR, -45 /* PCRE2_ERROR_JIT_BADOPTION: JIT not supported */))
+                        verbose_printerr("JITing regex \"%s\" failed: %s\n", pattern, error.message());
         }
 }
 
@@ -652,29 +643,20 @@ vteapp_search_popover_update_regex(VteappSearchPopover* popover)
         popover->regex_pattern = nullptr;
 
         if (search_text[0] != '\0') {
-                GError* error = nullptr;
+                auto error = vte::glib::Error{};
+                auto regex = compile_regex_for_search(pattern, caseless, error);
+                vte_terminal_search_set_regex(popover->terminal, regex, 0);
+                if (regex != nullptr)
+                        vte_regex_unref(regex);
 
-                if (options.use_gregex) {
-                        auto regex = compile_gregex(pattern, caseless, &error);
-                        vte_terminal_search_set_gregex(popover->terminal, regex, GRegexMatchFlags(0));
-                        if (regex != nullptr)
-                                g_regex_unref(regex);
-                } else {
-                        auto regex = compile_regex_for_search(pattern, caseless, &error);
-                        vte_terminal_search_set_regex(popover->terminal, regex, 0);
-                        if (regex != nullptr)
-                                vte_regex_unref(regex);
-                }
-
-                if (error == nullptr) {
+                if (error.error()) {
                         popover->has_regex = true;
                         popover->regex_pattern = pattern; /* adopt */
                         pattern = nullptr; /* adopted */
                         gtk_widget_set_tooltip_text(popover->search_entry, nullptr);
                 } else {
                         popover->has_regex = false;
-                        gtk_widget_set_tooltip_text(popover->search_entry, error->message);
-                        g_error_free(error);
+                        gtk_widget_set_tooltip_text(popover->search_entry, error.message());
                 }
         }
 
@@ -1009,26 +991,17 @@ vteapp_window_add_dingus(VteappWindow* window,
                          char const* const* dingus)
 {
         for (auto i = 0; dingus[i] != nullptr; i++) {
-                int tag = -1;
-                GError* error = nullptr;
-                if (options.use_gregex) {
-                        auto regex = compile_gregex(dingus[i], true, &error);
-                        if (regex) {
-                                tag = vte_terminal_match_add_gregex(window->terminal, regex,
-                                                                    GRegexMatchFlags(0));
-                                g_regex_unref(regex);
-                        }
-                } else {
-                        auto regex = compile_regex_for_match(dingus[i], true, &error);
-                        if (regex) {
-                                tag = vte_terminal_match_add_regex(window->terminal, regex, 0);
-                                vte_regex_unref(regex);
-                        }
+                auto tag = int{-1};
+                auto error = vte::glib::Error{};
+                auto regex = compile_regex_for_match(dingus[i], true, error);
+                if (regex) {
+                        tag = vte_terminal_match_add_regex(window->terminal, regex, 0);
+                        vte_regex_unref(regex);
                 }
 
-                if (error != nullptr) {
-                        verbose_printerr("Failed to compile regex \"%s\": %s\n", dingus[i], error->message);
-                        g_error_free(error);
+                if (error.error()) {
+                        verbose_printerr("Failed to compile regex \"%s\": %s\n",
+                                         dingus[i], error.message());
                 }
 
                 if (tag != -1)
@@ -1234,12 +1207,15 @@ vteapp_window_launch_argv(VteappWindow* window,
                           char** argv,
                           GError** error)
 {
+        auto const spawn_flags = GSpawnFlags(G_SPAWN_SEARCH_PATH_FROM_ENVP |
+                                             (options.no_systemd_scope ? VTE_SPAWN_NO_SYSTEMD_SCOPE : 0) |
+                                             (options.require_systemd_scope ? VTE_SPAWN_REQUIRE_SYSTEMD_SCOPE : 0));
         vte_terminal_spawn_async(window->terminal,
                                  VTE_PTY_DEFAULT,
                                  options.working_directory,
                                  argv,
                                  options.environment,
-                                 G_SPAWN_SEARCH_PATH_FROM_ENVP,
+                                 spawn_flags,
                                  nullptr, nullptr, nullptr, /* child setup, data and destroy */
                                  30 * 1000 /* 30s timeout */,
                                  nullptr /* cancellable */,
@@ -1329,22 +1305,20 @@ vteapp_window_fork(VteappWindow* window,
 static void
 vteapp_window_launch(VteappWindow* window)
 {
-        bool rv;
-        GError* err = nullptr;
+        auto rv = bool{};
+        auto error = vte::glib::Error{};
 
         if (options.exec_argv != nullptr)
-                rv = vteapp_window_launch_argv(window, options.exec_argv, &err);
+                rv = vteapp_window_launch_argv(window, options.exec_argv, error);
         else if (options.command != nullptr)
-                rv = vteapp_window_launch_commandline(window, options.command, &err);
+                rv = vteapp_window_launch_commandline(window, options.command, error);
         else if (!options.no_shell)
-                rv = vteapp_window_launch_shell(window, &err);
+                rv = vteapp_window_launch_shell(window, error);
         else
-                rv = vteapp_window_fork(window, &err);
+                rv = vteapp_window_fork(window, error);
 
-        if (!rv) {
-                verbose_printerr("Error launching: %s\n", err->message);
-                g_error_free(err);
-        }
+        if (!rv)
+                verbose_printerr("Error launching: %s\n", error.message());
 }
 
 static void
@@ -1491,32 +1465,23 @@ vteapp_window_show_context_menu(VteappWindow* window,
                 static const char extra_pattern[] = "(\\d+)\\s*(\\w+)";
                 char* extra_match = nullptr;
                 char *extra_subst = nullptr;
-                if (options.use_gregex) {
-                        auto regex = compile_gregex(extra_pattern, false, nullptr);
-                        vte_terminal_event_check_gregex_simple(window->terminal, event,
-                                                               &regex, 1,
-                                                               GRegexMatchFlags(0),
-                                                               &extra_match);
-                        g_regex_unref(regex);
-                } else {
-                        auto regex = compile_regex_for_match(extra_pattern, false, nullptr);
-                        vte_terminal_event_check_regex_simple(window->terminal, event,
-                                                              &regex, 1, 0,
-                                                              &extra_match);
+                auto error = vte::glib::Error{};
+                auto regex = compile_regex_for_match(extra_pattern, false, error);
+                error.assert_no_error();
+                vte_terminal_event_check_regex_simple(window->terminal, event,
+                                                      &regex, 1, 0,
+                                                      &extra_match);
 
-                        GError *err = nullptr;
-                        if (extra_match != nullptr &&
-                            (extra_subst = vte_regex_substitute(regex, extra_match, "$2 $1",
-                                                                PCRE2_SUBSTITUTE_EXTENDED |
-                                                                PCRE2_SUBSTITUTE_GLOBAL,
-                                                                &err)) == nullptr) {
-                                verbose_printerr("Substitution failed: %s\n", err->message);
-                                g_error_free(err);
-                        }
-
-                        vte_regex_unref(regex);
+                if (extra_match != nullptr &&
+                    (extra_subst = vte_regex_substitute(regex, extra_match, "$2 $1",
+                                                        PCRE2_SUBSTITUTE_EXTENDED |
+                                                        PCRE2_SUBSTITUTE_GLOBAL,
+                                                        error)) == nullptr) {
+                        verbose_printerr("Substitution failed: %s\n", error.message());
                 }
 
+                vte_regex_unref(regex);
+
                 if (extra_match != nullptr) {
                         if (extra_subst != nullptr)
                                 verbose_print("%s match: %s => %s\n", extra_pattern, extra_match, extra_subst);
@@ -1585,22 +1550,20 @@ window_child_exited_cb(VteTerminal* term,
 
         if (options.output_filename != nullptr) {
                 auto file = g_file_new_for_commandline_arg(options.output_filename);
-
-                GError* err = nullptr;
-                auto stream = g_file_replace(file, nullptr, false, G_FILE_CREATE_NONE, nullptr, &err);
+                auto error = vte::glib::Error{};
+                auto stream = g_file_replace(file, nullptr, false, G_FILE_CREATE_NONE, nullptr, error);
                 g_object_unref(file);
                 if (stream != nullptr) {
                         vte_terminal_write_contents_sync(window->terminal,
                                                          G_OUTPUT_STREAM(stream),
                                                          VTE_WRITE_DEFAULT,
-                                                         nullptr, &err);
+                                                         nullptr, error);
                         g_object_unref(stream);
                 }
 
-                if (err != nullptr) {
+                if (error.error()) {
                         verbose_printerr("Failed to write output to \"%s\": %s\n",
-                                         options.output_filename, err->message);
-                        g_error_free(err);
+                                         options.output_filename, error.message());
                 }
         }
 
@@ -1909,11 +1872,9 @@ vteapp_window_constructed(GObject *object)
         }
 
         if (options.encoding != nullptr) {
-                GError* err = nullptr;
-                if (!vte_terminal_set_encoding(window->terminal, options.encoding, &err)) {
-                        verbose_printerr("Failed to set encoding: %s\n", err->message);
-                        g_error_free(err);
-                }
+                auto error = vte::glib::Error{};
+                if (!vte_terminal_set_encoding(window->terminal, options.encoding, error))
+                        g_printerr("Failed to set encoding: %s\n", error.message());
         }
 
         if (options.word_char_exceptions != nullptr)
@@ -2198,10 +2159,9 @@ main(int argc,
        g_set_prgname("Terminal");
        g_set_application_name("Terminal");
 
-       GError* error = nullptr;
-       if (!options.parse_argv(argc, argv, &error)) {
-               g_printerr("Error parsing arguments: %s\n", error->message);
-               g_error_free(error);
+       auto error = vte::glib::Error{};
+       if (!options.parse_argv(argc, argv, error)) {
+               g_printerr("Error parsing arguments: %s\n", error.message());
                return EXIT_FAILURE;
        }
 
@@ -2209,7 +2169,7 @@ main(int argc,
                 verbose_printerr("VTE_CJK_WIDTH is not supported anymore, use --cjk-width instead\n");
 
        if (options.version) {
-               g_print("VTE Application %s\n", VERSION);
+               g_print("VTE Application %s %s\n", VERSION, vte_get_features());
                return EXIT_SUCCESS;
        }
 
